@@ -25,6 +25,7 @@
 #include <near/tag.h>
 
 struct near_ndef_record;
+struct carrier_data;
 
 struct near_ndef_message {
 	size_t length;
@@ -32,7 +33,15 @@ struct near_ndef_message {
 	uint8_t *data;
 };
 
-GList *near_ndef_parse(uint8_t *ndef_data, size_t ndef_length);
+int near_ndef_count_records(uint8_t *ndef_in, size_t ndef_in_length,
+						uint8_t record_type);
+
+int near_ndef_record_length(uint8_t *ndef_in, size_t ndef_in_length);
+
+GList *near_ndef_parse_msg(uint8_t *ndef_data, size_t ndef_length,
+					struct near_ndef_message **reply);
+
+void near_ndef_records_free(GList *records);
 
 struct near_ndef_message *near_ndef_prepare_text_record(char *encoding,
 					char *language_code, char *text);
@@ -40,9 +49,23 @@ struct near_ndef_message *near_ndef_prepare_text_record(char *encoding,
 struct near_ndef_message *near_ndef_prepare_uri_record(uint8_t identifier,
 					 uint32_t field_length, uint8_t *field);
 
+struct near_ndef_message *near_ndef_prepare_handover_record(char* type_name,
+					struct near_ndef_record *record,
+					uint8_t carriers,
+					struct carrier_data *remote);
+
 struct near_ndef_message *
 near_ndef_prepare_smartposter_record(uint8_t uri_identifier,
 					uint32_t uri_field_length,
 					uint8_t *uri_field);
 
+near_bool_t near_ndef_record_cmp_id(struct near_ndef_record *rec1,
+						struct near_ndef_record *rec2);
+near_bool_t near_ndef_record_cmp_mime(struct near_ndef_record *rec1,
+						struct near_ndef_record *rec2);
+
+
+size_t near_ndef_data_length(struct near_ndef_record *data);
+uint8_t *near_ndef_data_ptr(struct near_ndef_record *data);
+
 #endif
