@@ -29,8 +29,8 @@
 #include <sys/socket.h>
 
 #include <linux/socket.h>
-#include <linux/nfc.h>
 
+#include <near/nfc_copy.h>
 #include <near/plugin.h>
 #include <near/log.h>
 #include <near/types.h>
@@ -128,7 +128,7 @@ void snep_core_parse_handover_record(int client_fd, uint8_t *ndef,
 	 * Bugfix Android: Fix 'cr' instead of 'Hr'
 	 * Bug is in Google:HandoverManager.java:645
 	 */
-	if (strncmp((char *)(ndef + 9), "Hr", 2) == 0) {
+	if (nfc_data_length > 9 && strncmp((char *)(ndef + 9), "Hr", 2) == 0) {
 		DBG("Android 4.1.1 found !!!");
 		*(ndef + 9) = 'c';
 	}
@@ -826,6 +826,7 @@ int snep_core_init(void)
 void snep_core_exit(void)
 {
 	snep_validation_exit();
+	snep_exit();
 
 	g_hash_table_destroy(snep_client_hash);
 	snep_client_hash = NULL;
