@@ -42,6 +42,12 @@ static DBusConnection *connection = NULL;
 
 static GHashTable *adapter_hash;
 
+enum near_adapter_rf_mode {
+	NEAR_ADAPTER_RF_MODE_IDLE      = 0,
+	NEAR_ADAPTER_RF_MODE_INITIATOR = 1,
+	NEAR_ADAPTER_RF_MODE_TARGET    = 2
+};
+
 #define NEAR_ADAPTER_MODE_INITIATOR 0x1
 #define NEAR_ADAPTER_MODE_TARGET    0x2
 #define NEAR_ADAPTER_MODE_DUAL      0x3
@@ -53,7 +59,7 @@ struct near_adapter {
 	uint32_t idx;
 	uint32_t protocols;
 	uint32_t poll_mode;
-	uint32_t rf_mode;
+	enum near_adapter_rf_mode rf_mode;
 
 	near_bool_t powered;
 	near_bool_t polling;
@@ -97,6 +103,8 @@ static void free_adapter(gpointer data)
 
 	g_free(adapter->name);
 	g_free(adapter->path);
+	g_hash_table_destroy(adapter->tags);
+	g_hash_table_destroy(adapter->devices);
 	g_free(adapter);
 }
 
@@ -114,14 +122,40 @@ static void free_device(gpointer data)
 	__near_device_remove(device);
 }
 
+static char *rf_mode_to_string(struct near_adapter *adapter)
+{
+	switch (adapter->rf_mode) {
+	case NEAR_ADAPTER_RF_MODE_IDLE:
+		return "Idle";
+	case NEAR_ADAPTER_RF_MODE_INITIATOR:
+		return "Initiator";
+	case NEAR_ADAPTER_RF_MODE_TARGET:
+		return "Target";
+	}
+
+	return NULL;
+}
+
 static void polling_changed(struct near_adapter *adapter)
 {
 
 	near_dbus_property_changed_basic(adapter->path,
-				NFC_ADAPTER_INTERFACE, "Polling",
+					NFC_ADAPTER_INTERFACE, "Polling",
 					DBUS_TYPE_BOOLEAN, &adapter->polling);
 }
 
+static void rf_mode_changed(struct near_adapter *adapter)
+{
+	const char *rf_mode = rf_mode_to_string(adapter);
+
+	if (rf_mode == NULL)
+		return;
+
+	near_dbus_property_changed_basic(adapter->path,
+					NFC_ADAPTER_INTERFACE, "Mode",
+					DBUS_TYPE_STRING, &rf_mode);
+}
+
 static int adapter_start_poll(struct near_adapter *adapter)
 {
 	int err;
@@ -174,7 +208,6 @@ static void append_path(gpointer key, gpointer value, gpointer user_data)
 
 	dbus_message_iter_append_basic(iter, DBUS_TYPE_OBJECT_PATH,
 							&adapter->path);
-
 }
 
 void __near_adapter_list(DBusMessageIter *iter, void *user_data)
@@ -192,36 +225,31 @@ static void append_protocols(DBusMessageIter *iter, void *user_data)
 	if (adapter->protocols & NFC_PROTO_FELICA_MASK) {
 		str = "Felica";
 
-		dbus_message_iter_append_basic(iter,
-				DBUS_TYPE_STRING, &str);
+		dbus_message_iter_append_basic(iter, DBUS_TYPE_STRING, &str);
 	}
 
 	if (adapter->protocols & NFC_PROTO_MIFARE_MASK) {
 		str = "MIFARE";
 
-		dbus_message_iter_append_basic(iter,
-				DBUS_TYPE_STRING, &str);
+		dbus_message_iter_append_basic(iter, DBUS_TYPE_STRING, &str);
 	}
 
 	if (adapter->protocols & NFC_PROTO_JEWEL_MASK) {
 		str = "Jewel";
 
-		dbus_message_iter_append_basic(iter,
-				DBUS_TYPE_STRING, &str);
+		dbus_message_iter_append_basic(iter, DBUS_TYPE_STRING, &str);
 	}
 
 	if (adapter->protocols & NFC_PROTO_ISO14443_MASK) {
 		str = "ISO-DEP";
 
-		dbus_message_iter_append_basic(iter,
-				DBUS_TYPE_STRING, &str);
+		dbus_message_iter_append_basic(iter, DBUS_TYPE_STRING, &str);
 	}
 
 	if (adapter->protocols & NFC_PROTO_NFC_DEP_MASK) {
 		str = "NFC-DEP";
 
-		dbus_message_iter_append_basic(iter,
-				DBUS_TYPE_STRING, &str);
+		dbus_message_iter_append_basic(iter, DBUS_TYPE_STRING, &str);
 	}
 }
 
@@ -237,8 +265,7 @@ static void append_tag_path(gpointer key, gpointer value, gpointer user_data)
 
 	DBG("%s", tag_path);
 
-	dbus_message_iter_append_basic(iter, DBUS_TYPE_OBJECT_PATH,
-							&tag_path);
+	dbus_message_iter_append_basic(iter, DBUS_TYPE_OBJECT_PATH, &tag_path);
 }
 
 static void append_tags(DBusMessageIter *iter, void *user_data)
@@ -282,14 +309,14 @@ void __near_adapter_tags_changed(uint32_t adapter_idx)
 	DBG("");
 
 	adapter = g_hash_table_lookup(adapter_hash,
-				GINT_TO_POINTER(adapter_idx));
+					GINT_TO_POINTER(adapter_idx));
 	if (adapter == NULL)
 		return;
 
 	near_dbus_property_changed_array(adapter->path,
-				NFC_ADAPTER_INTERFACE, "Tags",
-				DBUS_TYPE_OBJECT_PATH, append_tags,
-				adapter);
+					NFC_ADAPTER_INTERFACE, "Tags",
+					DBUS_TYPE_OBJECT_PATH, append_tags,
+					adapter);
 }
 
 void __near_adapter_devices_changed(uint32_t adapter_idx)
@@ -299,20 +326,21 @@ void __near_adapter_devices_changed(uint32_t adapter_idx)
 	DBG("");
 
 	adapter = g_hash_table_lookup(adapter_hash,
-				GINT_TO_POINTER(adapter_idx));
+					GINT_TO_POINTER(adapter_idx));
 	if (adapter == NULL)
 		return;
 
 	near_dbus_property_changed_array(adapter->path,
-				NFC_ADAPTER_INTERFACE, "Devices",
-				DBUS_TYPE_OBJECT_PATH, append_devices,
-				adapter);
+					NFC_ADAPTER_INTERFACE, "Devices",
+					DBUS_TYPE_OBJECT_PATH, append_devices,
+					adapter);
 }
 
 static DBusMessage *get_properties(DBusConnection *conn,
 					DBusMessage *msg, void *data)
 {
 	struct near_adapter *adapter = data;
+	const char *rf_mode;
 	DBusMessage *reply;
 	DBusMessageIter array, dict;
 
@@ -332,6 +360,11 @@ static DBusMessage *get_properties(DBusConnection *conn,
 	near_dbus_dict_append_basic(&dict, "Polling",
 				    DBUS_TYPE_BOOLEAN, &adapter->polling);
 
+	rf_mode = rf_mode_to_string(adapter);
+	if (rf_mode != NULL)
+		near_dbus_dict_append_basic(&dict, "Mode",
+						DBUS_TYPE_STRING, &rf_mode);
+
 	near_dbus_dict_append_array(&dict, "Protocols",
 				DBUS_TYPE_STRING, append_protocols, adapter);
 
@@ -375,15 +408,20 @@ static DBusMessage *set_property(DBusConnection *conn,
 
 		err = __near_netlink_adapter_enable(adapter->idx, powered);
 		if (err < 0) {
-			if (err == -EALREADY)
-				return __near_error_already_enabled(msg);
+			if (err == -EALREADY) {
+				if (powered == TRUE)
+					return __near_error_already_enabled(msg);
+				else
+					return __near_error_already_disabled(msg);
+			}
 
 			return __near_error_failed(msg, -err);
 		}
 
 		adapter->powered = powered;
-	} else
+	} else {
 		return __near_error_invalid_property(msg);
+	}
 
 	return g_dbus_create_reply(msg, DBUS_TYPE_INVALID);
 }
@@ -426,6 +464,9 @@ static DBusMessage *stop_poll_loop(DBusConnection *conn,
 
 	DBG("conn %p", conn);
 
+	if (adapter->polling == FALSE)
+		return __near_error_not_polling(msg);
+
 	err = __near_netlink_stop_poll(adapter->idx);
 	if (err < 0)
 		return __near_error_failed(msg, -err);
@@ -498,6 +539,39 @@ static void tag_present_cb(uint32_t adapter_idx, uint32_t target_idx,
 					check_presence, adapter);
 }
 
+void __near_adapter_start_check_presence(uint32_t adapter_idx,
+						uint32_t target_idx)
+{
+	struct near_adapter *adapter;
+
+	DBG("");
+
+	adapter = g_hash_table_lookup(adapter_hash,
+			GINT_TO_POINTER(adapter_idx));
+	if (adapter == NULL)
+		return;
+
+	adapter->presence_timeout =
+			g_timeout_add_seconds(CHECK_PRESENCE_PERIOD,
+					check_presence, adapter);
+}
+
+void __near_adapter_stop_check_presence(uint32_t adapter_idx,
+						uint32_t target_idx)
+{
+	struct near_adapter *adapter;
+
+	DBG("");
+
+	adapter = g_hash_table_lookup(adapter_hash,
+			GINT_TO_POINTER(adapter_idx));
+	if (adapter == NULL)
+		return;
+
+	if (adapter->presence_timeout > 0)
+		g_source_remove(adapter->presence_timeout);
+}
+
 static const GDBusMethodTable adapter_methods[] = {
 	{ GDBUS_METHOD("GetProperties",
 				NULL, GDBUS_ARGS({"properties", "a{sv}"}),
@@ -519,7 +593,7 @@ static const GDBusSignalTable adapter_signals[] = {
 	{ }
 };
 
-struct near_adapter * __near_adapter_create(uint32_t idx,
+struct near_adapter *__near_adapter_create(uint32_t idx,
 		const char *name, uint32_t protocols, near_bool_t powered)
 {
 	struct near_adapter *adapter;
@@ -583,9 +657,31 @@ int __near_adapter_set_dep_state(uint32_t idx, near_bool_t dep)
 	if (dep == FALSE && adapter->constant_poll == TRUE)
 		adapter_start_poll(adapter);
 
+	if (dep == FALSE) {
+		uint32_t target_idx;
+
+		target_idx =  __neard_device_get_idx(adapter->device_link);
+		__near_adapter_remove_target(idx, target_idx);
+	} else {
+		__near_adapter_devices_changed(idx);
+	}
+
 	return 0;
 }
 
+near_bool_t __near_adapter_get_dep_state(uint32_t idx)
+{
+	struct near_adapter *adapter;
+
+	DBG("idx %d", idx);
+
+	adapter = g_hash_table_lookup(adapter_hash, GINT_TO_POINTER(idx));
+	if (adapter == NULL)
+		return FALSE;
+
+	return adapter->dep_up;
+}
+
 int __near_adapter_add(struct near_adapter *adapter)
 {
 	uint32_t idx = adapter->idx;
@@ -602,7 +698,7 @@ int __near_adapter_add(struct near_adapter *adapter)
 	g_dbus_register_interface(connection, adapter->path,
 					NFC_ADAPTER_INTERFACE,
 					adapter_methods, adapter_signals,
-							NULL, adapter, NULL);
+					NULL, adapter, NULL);
 
 	return 0;
 }
@@ -666,11 +762,8 @@ static void device_read_cb(uint32_t adapter_idx, uint32_t target_idx,
 
 		return;
 	}
-
-	__near_adapter_devices_changed(adapter_idx);
 }
 
-
 static int adapter_add_tag(struct near_adapter *adapter, uint32_t target_idx,
 			uint32_t protocols,
 			uint16_t sens_res, uint8_t sel_res,
@@ -686,8 +779,7 @@ static int adapter_add_tag(struct near_adapter *adapter, uint32_t target_idx,
 	if (tag == NULL)
 		return -ENODEV;
 
-	g_hash_table_insert(adapter->tags, GINT_TO_POINTER(target_idx),
-								tag);
+	g_hash_table_insert(adapter->tags, GINT_TO_POINTER(target_idx), tag);
 
 	tag_type = __near_tag_get_type(tag);
 
@@ -697,7 +789,15 @@ static int adapter_add_tag(struct near_adapter *adapter, uint32_t target_idx,
 		return err;
 	}
 
-	return __near_tag_read(tag, tag_read_cb);
+	err = __near_tag_read(tag, tag_read_cb);
+	if (err < 0) {
+		near_error("Could not read the tag");
+
+		near_adapter_disconnect(adapter->idx);
+		__near_adapter_remove_target(adapter->idx, target_idx);
+	}
+
+	return err;
 }
 
 static int adapter_add_device(struct near_adapter *adapter,
@@ -750,13 +850,15 @@ int __near_adapter_add_target(uint32_t idx, uint32_t target_idx,
 	adapter->polling = FALSE;
 	polling_changed(adapter);
 
+	adapter->rf_mode = NEAR_ADAPTER_RF_MODE_INITIATOR;
+	rf_mode_changed(adapter);
+
 	if (protocols & NFC_PROTO_NFC_DEP_MASK)
 		return adapter_add_device(adapter, target_idx,
 						nfcid, nfcid_len);
 	else
 		return adapter_add_tag(adapter, target_idx, protocols,
 					sens_res, sel_res, nfcid, nfcid_len);
-
 }
 
 int __near_adapter_remove_target(uint32_t idx, uint32_t target_idx)
@@ -769,15 +871,18 @@ int __near_adapter_remove_target(uint32_t idx, uint32_t target_idx)
 	if (adapter == NULL)
 		return -ENODEV;
 
+	adapter->rf_mode = NEAR_ADAPTER_RF_MODE_IDLE;
+	rf_mode_changed(adapter);
+
 	if (g_hash_table_remove(adapter->tags,
-			GINT_TO_POINTER(target_idx)) == TRUE) {      	
+			GINT_TO_POINTER(target_idx)) == TRUE) {
 		__near_adapter_tags_changed(idx);
 
 		return 0;
 	}
 
 	if (g_hash_table_remove(adapter->devices,
-			GINT_TO_POINTER(target_idx)) == TRUE) {      	
+			GINT_TO_POINTER(target_idx)) == TRUE) {
 		__near_adapter_devices_changed(idx);
 
 		return 0;
@@ -789,6 +894,7 @@ int __near_adapter_remove_target(uint32_t idx, uint32_t target_idx)
 int __near_adapter_add_device(uint32_t idx, uint8_t *nfcid, uint8_t nfcid_len)
 {
 	struct near_adapter *adapter;
+	int ret;
 
 	DBG("idx %d", idx);
 
@@ -798,9 +904,17 @@ int __near_adapter_add_device(uint32_t idx, uint8_t *nfcid, uint8_t nfcid_len)
 
 	adapter->polling = FALSE;
 	adapter->dep_up = TRUE;
+	adapter->rf_mode = NEAR_ADAPTER_RF_MODE_TARGET;
 	polling_changed(adapter);
+	rf_mode_changed(adapter);
 
-	return adapter_add_device(adapter, 0, nfcid, nfcid_len);
+	ret = adapter_add_device(adapter, 0, nfcid, nfcid_len);
+	if (ret < 0)
+		return ret;
+
+	__near_adapter_devices_changed(idx);
+
+	return 0;
 }
 
 int __near_adapter_remove_device(uint32_t idx)
@@ -818,6 +932,8 @@ int __near_adapter_remove_device(uint32_t idx)
 			GINT_TO_POINTER(device_idx)) == FALSE)
 		return 0;
 
+	adapter->rf_mode = NEAR_ADAPTER_RF_MODE_IDLE;
+	rf_mode_changed(adapter);
 	__near_adapter_devices_changed(idx);
 
 	adapter->dep_up = FALSE;
@@ -968,7 +1084,7 @@ int near_adapter_disconnect(uint32_t idx)
 		return -ENOLINK;
 
 	tag_type = __near_tag_get_type(adapter->tag_link);
-	target_idx = __near_tag_get_idx(adapter->tag_link);
+	target_idx = near_tag_get_target_idx(adapter->tag_link);
 
 	DBG("tag type %d", tag_type);
 
@@ -991,7 +1107,7 @@ int near_adapter_disconnect(uint32_t idx)
 }
 
 int near_adapter_send(uint32_t idx, uint8_t *buf, size_t length,
-			near_recv cb, void *data)
+			near_recv cb, void *data, near_release data_rel)
 {
 	struct near_adapter *adapter;
 	struct near_adapter_ioreq *req = NULL;
@@ -1000,20 +1116,26 @@ int near_adapter_send(uint32_t idx, uint8_t *buf, size_t length,
 	DBG("idx %d", idx);
 
 	adapter = g_hash_table_lookup(adapter_hash, GINT_TO_POINTER(idx));
-	if (adapter == NULL)
-		return -ENODEV;
+	if (adapter == NULL) {
+		err = -ENODEV;
+		goto out_err;
+	}
 
-	if (adapter->tag_sock == -1 || adapter->tag_link == NULL)
-		return -ENOLINK;
+	if (adapter->tag_sock == -1 || adapter->tag_link == NULL) {
+		err = -ENOLINK;
+		goto out_err;
+	}
 
 	if (cb != NULL && adapter->watch != 0) {
 		req = g_try_malloc0(sizeof(*req));
-		if (req == NULL)
-			return -ENOMEM;
+		if (req == NULL) {
+			err = -ENOMEM;
+			goto out_err;
+		}
 
 		DBG("req %p cb %p data %p", req, cb, data);
 
-		req->target_idx = __near_tag_get_idx(adapter->tag_link);
+		req->target_idx = near_tag_get_target_idx(adapter->tag_link);
 		req->cb = cb;
 		req->data = data;
 
@@ -1036,6 +1158,9 @@ out_err:
 				g_list_delete_link(adapter->ioreq_list, last);
 	}
 
+	if (data_rel != NULL)
+		return (*data_rel)(err, data);
+
 	return err;
 }
 
@@ -1052,7 +1177,6 @@ static void adapter_listen(gpointer key, gpointer value, gpointer user_data)
 	driver->listen(adapter->idx, device_read_cb);
 }
 
-
 void __near_adapter_listen(struct near_device_driver *driver)
 {
 	g_hash_table_foreach(adapter_hash, adapter_listen, driver);
