@@ -202,12 +202,17 @@ int main(int argc, char *argv[])
 
 	parse_config(config);
 
-	__near_netlink_init();
+	if (__near_netlink_init() < 0) {
+		near_error("*** NETLINK INITIALIZATION FAILED ***");
+		exit(1);
+	}
+
 	__near_tag_init();
 	__near_device_init();
 	__near_adapter_init();
 	__near_ndef_init();
 	__near_manager_init(conn);
+	__near_agent_init();
 	__near_bluetooth_init();
 
 	__near_plugin_init(option_plugin, option_noplugin);
@@ -221,6 +226,7 @@ int main(int argc, char *argv[])
 
 	__near_plugin_cleanup();
 
+	__near_agent_cleanup();
 	__near_bluetooth_cleanup();
 	__near_manager_cleanup();
 	__near_ndef_cleanup();
